/**
 * Controller object to handle list-type tab selection.
 */
function TabController()
{
	// Tab classes to indicate de/selection.
	var _selTabClass = "";
	var _deselTabClass = "";

	// Tab content classes to indicate de/selection.
	var _selContentClass = "";
	var _deselContentClass = "";
	
	// Map containing the tab ID (String) and the content ID (String).
	var _tabMap = new Object();

	var _currSelTab;
	var _currSelContent;

	/**
	 * Set the CSS classes to use by the tabs and contents.
	 *
	 * @param classes (Array) the specified CSS classes. It must be of the form: [tab_selected,
	 *        tab_deselected, content_selected, content_deselected].
	 */
	this.setClasses = function( classes )
	{
		_selTabClass = classes[0];
		_deselTabClass = classes[1];
		_selContentClass = classes[2];
		_deselContentClass = classes[3];
	}

	/**
	 * Register a tab with the associated content.
	 *
	 * @param tabId (String) the ID of the tab to register.
	 * @param contentId (String) the ID of the corresponding content.
	 */
	this.registerTab = function( tabId, contentId )
	{
		if ( document.getElementById( tabId ).className == _selTabClass )
		{
			_currSelTab = tabId;
			_currSelContent = contentId;
		}
	
		_tabMap[ tabId ] = contentId;
	}

	/**
	 * Show the content for the current selected tab.
	 *
	 * @param src (Object) the current selected tab.
	 */
	this.showContent = function( src )
	{
		var tabId = src.id;
	
		if ( _currSelTab == tabId )
		{
			// Nothing to do.
			return;
		}
		
		// Hide previously selected tab and content.
		document.getElementById( _currSelTab ).className = _deselTabClass;
		document.getElementById( _currSelContent ).className = _deselContentClass;
		
		// Show current selected tab and content.
		document.getElementById( tabId ).className = _selTabClass;
		document.getElementById( _tabMap[tabId] ).className = _selContentClass;

		_currSelTab = tabId;
		_currSelContent = _tabMap[tabId];
	}
}

// -------------------------

/**
 * Controller object to handle events for the Select list-DIV widget.
 *
 * @param selectList (HTMLSelectElement) the Select list to handle.
 */
function SelectDivController( selectList )
{
	var _selectList = selectList;
	
	// Map containing the option value (key) and the associated content ID (value).
	var _optMap = new Object();
	
	// Select option content classes to indicate de/selection.
	var _deselContentClass = "";
	var _deselContentClass = "";

	var _currSelContentId;

	/**
	 * Set the CSS classes to use by the content DIVs.
	 *
	 * @param classes (Array) the specified CSS classes. It must be of the form: [ content_selected,
	 *        content_deselected].
	 */
	this.setClasses = function( classes )
	{
		_selContentClass = classes[0];
		_deselContentClass = classes[1];
	}

	/**
	 * Register a Select list option value with its associated DIV content ID.
	 *
	 * @param optValue (String) the option value to register.
	 * @param contentId (String) the associated DIV content ID.
	 */
	this.registerOption = function( optValue, contentId )
	{
		if ( document.getElementById( contentId ).className == _selContentClass )
		{
			_currSelContentId = contentId;
		}

		_optMap[optValue] = contentId;
	}

	/**
	 * Show the DIV content corresponding to the current selected option.
	 */
	this.showContent = function()
	{
		if ( _selectList.selectedIndex == -1 )
			return;

		var selContentId = _optMap[_selectList.value];
		if ( selContentId == null )
			return;
		
		// Hide previously selected content.
		document.getElementById( _currSelContentId ).className = _deselContentClass;
		
		// Show current selected content.
		document.getElementById( selContentId ).className = _selContentClass;
		
		_currSelContentId = selContentId;
	}	
}

// -------------------------

/**
 * Controller object to handle events for the dual-list widget.
 *
 * @param availableListId (String) a Select list containing elements from which to transfer.
 *        Must be sorted in ascending order.
 * @param selectedListId (String) a Select list to which elements are to be transferred to.
 *        Must be sorted in ascending order.
 */
function DualListController( availableListId, selectedListId )
{
	/** Available (source) list. */
	var _availList = document.getElementById( availableListId );

	/** Selected (destination) list. */
	var _selList = document.getElementById( selectedListId );

	var _observer = null;

	var _guiController = null;

	var _availListEmpty = false;

	var _selListEmpty = false;

	var _toggler = new ToggleController();

	// Default comparator.
	var _comparator = new Function( "arg0", "arg1",
		"if ( arg0 < arg1 ) return -1; else if ( arg0 == arg1 ) return 0; else return 1;" );

	/**
	 * Initialise the GUI.
	 */
	this.initGUI = function()
	{
		if ( _availList.length < 1 )
		{
			_availListEmpty = true;
			_toggler.toggleAll( _availList.id );
		}
		else if ( _selList.length < 1 )
		{
			_selListEmpty = true;
			_toggler.toggleAll( _selList.id );
		}
	}
	
	/**
	 * Set an Observer to listen for "form-dirtying" actions. The Observer must implement a
	 * setDirty(boolean) method.
	 *
	 * @param observer (Object) The specified Observer.
	 */
	this.setDirtyObserver = function( observer )
	{
		_observer = observer;
	}

	/**
	 * Set the dual-list buttons whose enabled states depend on the states of the Select lists.
	 *
	 * @param buttonIds (Array) The button IDs, arranged in the following order: [addItem,
	 *        addAllItems, removeItem, removeAllItems]. Specify null for buttons that are not to be
	 *        set.
	 */
	this.setButtons = function( buttonIds )
	{
		for ( var i = 0; i < buttonIds.length; i++ )
		{
			if ( i < 2 ) addButtonDisabledToggler( _availList.id, buttonIds[i] );
			else addButtonDisabledToggler( _selList.id, buttonIds[i] );
		}
	}

	/**
	 * Add a "disabled" property Toggler to the ToggleController for a button.
	 *
	 * @param listId (String) The Select list ID that will affect the "disabled" state of the
	 *        button.
	 * @param buttonId (String) The button ID whose "disabled" state will be toggled.
	 */
	function addButtonDisabledToggler( listId, buttonId )
	{
		if ( buttonId == null ) return;

		_toggler.addToggler( listId, new AttributeToggler( buttonId, "disabled", true ) );
	}

	/**
	 * Add a toggler for an external component that toggles based on the Selected list state.
	 *
	 * @param toggler (Toggler) The Toggler Object (that implements a toggle() method) to add.
	 */
	this.addSelListToggler = function( toggler )
	{
		_toggler.addToggler( _selList.id, toggler );
	}

	/**
	 * Set a compare function to determine the order in which elements from one list is to be
	 * inserted into the other. If not set, the default compare function uses the elements' natural
	 * sort order.
	 *
	 * @param compareFunc (Function) The specified compare function. This function must accept two
	 *        string arguments - a and b - and returns < 0 if a < b, 0 if a == b, and > 0 if a > b.
	 */
	this.setCompareFunction = function( compareFunc )
	{
		_comparator = compareFunc;
	}

	/**
	 * Handler method to add option(s) from the Available list to the Selected list.
	 */
	this.addItem = function()
	{
		transferSelected( _availList, _selList );
		setGUIAddItem();
	}

	/**
	 * Handler method to add all options from the Available list to the Selected list.
	 */
	this.addAllItems = function()
	{
		transferAll( _availList, _selList );
		setGUIAddItem();
	}

	/**
	 * Handler method to move option(s) from the Selected list back to the Available list.
	 */
	this.removeItem = function()
	{
		transferSelected( _selList, _availList );
		setGUIRemoveItem();
	}

	/**
	 * Handler method to add all options from the Selected list back to the Available list.
	 */
	this.removeAllItems = function()
	{
		transferAll( _selList, _availList );
		setGUIRemoveItem();
	}

	function setGUIAddItem()
	{
		// If Available list is now empty...
		if ( _availList.length < 1 )
		{
			_availListEmpty = true;
			_toggler.toggleAll( _availList.id );
		}

		// If Selected list was empty...
		if ( _selListEmpty && _selList.length > 0 )
		{
			_selListEmpty = false;
			_toggler.toggleAll( _selList.id );
		}

		//APAR:IZ22028-Added to set the width in both available as well as selected list boxes 
		//according to the length of the item on adding items from available to selected list respectively.
		set_width(_availList,_selList);
	}

	function setGUIRemoveItem()
	{
		// If Selected list is now empty...
		if ( _selList.length < 1 )
		{
			_selListEmpty = true;
			_toggler.toggleAll( _selList.id );
		}

		// If Available list was empty...
		if ( _availListEmpty && _availList.length > 0 )
		{
			_availListEmpty = false;
			_toggler.toggleAll( _availList.id );
		}

		//APAR:IZ22028-Added to set the width in both available as well as selected list boxes 
		//according to the length of the item on removing items from selected to available list respectively.
		set_width(_availList, _selList);
	}

	/**
	 * Transfer selected elements from the source list to the destination list.
	 *
	 * @param srcList (HTMLSelectElement) The source list.
	 * @param destList (HTMLSelectElement) The destination list.
	 */
	function transferSelected( srcList, destList )
	{
		if ( destList.length < 1 )
		{
			while ( srcList.selectedIndex != -1 )
			{
				destList.appendChild( srcList.options.item( srcList.selectedIndex ) );
			}
		}
		else
		{
			while ( srcList.selectedIndex != -1 )
			{
				transferOption( srcList.selectedIndex, srcList, destList );
			}
		}
	}

	/**
	 * Transfer all element from the source list to the destination list.
	 *
	 * @param srcList (HTMLSelectElement) The source Select list.
	 * @param destList (HTMLSelectElement) The destination Select list.
	 */
	function transferAll( srcList, destList )
	{
		if ( destList.length < 1 )
		{
			while ( srcList.length > 0 )
			{
				destList.appendChild( srcList.options.item( 0 ) );
			}
		}
		else
		{
			while ( srcList.length > 0 )
			{
				transferOption( 0, srcList, destList );
			}
		}
	}

	/**
	 * Transfer elements from the source list to the destination list by appending each element from
	 * the source list to the destination list.
	 *
	 * @param srcList (HTMLSelectElement) The source Select list.
	 * @param destList (HTMLSelectElement) The destination Select list.
	 */
	function transferAppend( srcList, destList )
	{
		while ( srcList.length > 0 )
		{
			destList.appendChild( srcList.options.item( 0 ) );
		}
	}
	
	/**
	 * Transfer an option from the source Select list to the destination Select list. Note that this
	 * has a side-effect of altering the list options' index as well as the lists' size.
	 *
	 * @param optIndex (int) The index of the option to transfer.
	 * @param srcList (HTMLSelectElement) the source list from which to transfer the option.
	 * @param destList (HTMLSelectElement) the destination list to which to transfer the option.
	 */
	function transferOption( optIndex, srcList, destList )
	{
		if ( optIndex < 0 )
			return;

		// Get the index to insert to.
		// Note: Use binary insertion sort rather than a more general-purpose sorting algorithm for
		// its simplicity, and to exploit the fact that both lists are already sorted to begin with.
		var option = srcList.options.item( optIndex );
		var insertIndex = getInsertIndex( option, destList );

		// Add to destination list.
		var selBeforeElem = destList.options.item( insertIndex );

		// Null selBeforeElem will simply cause the element to be added to the end.
		// IE workarounds: Must use insertBefore() because IE6 doesn't support W3C's DOM-HTML add(),
		// and use innerText otherwise Option label will not appear in the Select list.
		option.innerText = option.text;
		destList.insertBefore( option, selBeforeElem );

		// Option(s) transferred, so form is dirty.
		if ( _observer != null ) _observer.setDirty( true );
	}

	/**
	 * Get the index to insert the Option element in the destination Select list.
	 *
	 * @param optionElem (HTMLOptionElement) the Option element to insert.
	 * @param destList (HTMLSelectElement) the destination Select list.
	 * @return The index before which the element is to be inserted.
	 */
	function getInsertIndex( optionElem, destList )
	{
		var lowIndex = 0;
		var highIndex = destList.length;

		while ( lowIndex < highIndex )
		{
			var midIndex = (lowIndex + highIndex) >> 1;
			var midValue = destList.options.item( midIndex ).text;

			if ( _comparator( optionElem.text, midValue ) >= 0 )
			{
				lowIndex = midIndex + 1;
			}
			else
			{
				highIndex = midIndex;
			}
		}

		return lowIndex;
	}
}

// -------------------------

/**
 * Controller object to handle enabling/disabling of components based on the checked status of
 * checkboxes or radio buttons.
 */
function DisabledStatusController()
{
	//Textfield classes defined in styles.css
	var _textfieldDisabled = "textfield-disabled";
	var _textfieldDefault = "textfield-default";


	// Map containing monitored component (checkbox or radio button) and the component to be
	// enabled/disabled.
	var _compMap = new Object();
	
	// Map containing monitored component and  the check/uncheck status that will disable the
	// associated component.
	var _disableMap = new Object();
	
	/**
	 * Register the component to be monitored, and the corresponding component whose disabled status
	 * is to be controlled.
	 *
	 * @param monitoredCompId (String) The component ID to monitor; must be either a checkbox or a
	 *        radio button, otherwise the component will be ignored.
	 * @param disableCompId (String) The form component (INPUT, SELECT) ID whose disabled status
	 *        is to be controlled.
	 * @param disable (boolean) Set to true if unchecking the monitored component will disable the
	 *        associated component; false if vice versa.
	 */
	this.registerComponent = function( monitoredCompId, disableCompId, disable )
	{
		if ( isCheckboxOrRadio( monitoredCompId ) )
		{
			_compMap[monitoredCompId] = disableCompId;
			_disableMap[monitoredCompId] = disable;
		}
	}

	/**
	 * Change the component disabled status.
	 *
	 * @param srcId (String) The current clicked component ID. The component must have been
	 *        registered.
	 */
	this.doDisable = function( srcId )
	{
		var disableComp = document.getElementById( _compMap[srcId] );
		if ( disableComp == undefined ) return;

		var checked = document.getElementById( srcId ).checked;
		var disable = _disableMap[srcId];

		disableComp.disabled = (!checked && disable) || (checked && !disable);
		if(disableComp.disabled)
			disableComp.className = _textfieldDisabled;
		else
			disableComp.className = _textfieldDefault;
	}
	
	/**
	 * Check if the form object is a checkbox or a radio button.
	 *
	 * @param formObjId (String) ID of the object to be checked.
	 * @return true if object is checkbox or radio button.
	 */	
	function isCheckboxOrRadio( formObjId )
	{
		var formObj = document.getElementById( formObjId );

		return ( formObj.nodeName == "INPUT"
			&& ( formObj.type == "checkbox" || formObj.type == "radio" ) );
	}
}

// -------------------------

/**
 * Controller object that executes Toggler Objects. Toggler Objects are Objects that implement the
 * "toggle()" method. This method does the actual toggling.
 */
function ToggleController()
{
	// Map containing the source ID and an array of Togglers. */
	var _togglerMap = new Object();

	/**
	 * Add a Toggler Object for a source Object.
	 *
	 * @param srcId (String) the ID of the source Object that initiates the toggling.
	 * @param toggler (Toggler) The Toggler Object (that implements a toggle() method) to add.
	 */
	this.addToggler = function( srcId, toggler )
	{
		var togglerArray = _togglerMap[srcId];
		if ( togglerArray == undefined )
		{
			togglerArray = new Array();
			_togglerMap[srcId] = togglerArray;
		}

		togglerArray.push( toggler );
	}

	/**
	 * Toggle all the Toggler Objects associated to a source Object.
	 *
	 * @param srcId (String) The ID of the source Object that initiated the toggling.
	 */
	this.toggleAll = function( srcId )
	{
		var togglerArray = _togglerMap[srcId];
		if ( togglerArray == undefined )
			return;

		// For each Toggler...
		for ( var i = 0; i < togglerArray.length; i++ )
		{
			togglerArray[i].toggle();
		}
	}
}

/**
 * Toggler Object that handles the toggling of a component attribute value.
 *
 * @param destId (String) The ID of the destination component to be toggled.
 * @param attribute (String) The component attribute whose value is to be toggled.
 * @param toggleAttrValue (String) The attribute value to toggle to.
 */
function AttributeToggler( destId, attribute, toggleAttrValue )
{
	var _destObj = document.getElementById( destId );
	var _attr = attribute;
	var _toggleAttrValue = toggleAttrValue;

	this.toggle = function()
	{
		var currentAttrValue = eval( "_destObj." + _attr );

		eval( "_destObj." + _attr + " = _toggleAttrValue" );
		_toggleAttrValue = currentAttrValue;
	}
}

// -------------------------

/**
 * Represents a server validation error.
 *
 * @param labelId (String) The label ID corresponding to the form field where the error occurred.
 * @param errorMsg (String) The error message.
 * @param errorStyleClass (String) The CSS style applied to the element. 
 *        Might not be defined, in which case the ValidationController#_dfltErrorClass is applied
 */
function ServerValidationError( labelId, errorMsg, errorStyleClass )
{
	var _labelId = labelId;
	var _errorMsg = errorMsg;
	var _errorStyleClass = errorStyleClass

	this.getLabelId = function()
	{
		return _labelId;
	}

	this.getErrorMsg = function()
	{
		return _errorMsg;
	}

	this.getErrorStyleClass = function()
	{
      	return _errorStyleClass;
	}
}

/**
 * Controller object to handle client-side validation.
 *
 * @param errorsSectionId (String) ID of section (div, p, etc.) containing error messages.
 * @param showErrorsClass (String) CSS class to use to show error messages in the errors section.
 * @param hideErrorsClass (String) CSS class to use to hide error messages in the errors section.
 * @param errorHeader (String) The header message to display in the error section.
 * @param messageIconPath (String) The path to the icon to be used in the message section.
 *        The default is set to "../styles/images/messages/statusCritical.png" which might not always apply.
 *        Therefore it might need to be set to a absolute path, e.g. when used within portlets.
 */
function ValidationController( errorsSectionId, showErrorsClass, hideErrorsClass, errorHeader, messageIconPath )
{
	var _errorsSection = document.getElementById( errorsSectionId );
	var _showErrors = showErrorsClass;
	var _hideErrors = hideErrorsClass;
	var _errorInstr = errorHeader;

	var _messageIconPath = "../styles/images/messages/statusCritical.png";
	if (messageIconPath != null)
	{
		_messageIconPath = messageIconPath;
	}
	
	// Validation rules.
	var _rules = new Array();

	// Error messages.
	var _errorMsgs = new Array();

	// Default CSS class to append to label for server validation errors.
	var _dfltErrorClass = "labelError";
	
	// Server validation errors.
	var _serverErrors = new Array();

	/**
	 * Inject server validation errors into this controller. The injected errors are transient,
	 * i.e., references to these errors are removed once they have been displayed, and therefore
	 * will not reappear on subsequent client-side validation.
	 *
	 * @param errors (Array) An array of ServerValidationError Objects.
	 */
	this.injectServerErrors = function( errors )
	{
		_serverErrors = errors;
	}

	/**
	 * Display the injected server validation errors.
	 */
	this.displayServerErrors = function()
	{
		for ( var i = 0; i < _serverErrors.length; i++ )
		{
			var error = _serverErrors[i];

			_errorMsgs.push( error.getErrorMsg() );
			var errorStyleClass = error.getErrorStyleClass();
			if (errorStyleClass == undefined || errorStyleClass == null)
			{
				errorStyleClass = _dfltErrorClass;
			}
			setErrorClass( document.getElementById( error.getLabelId() ), errorStyleClass );
		}
		
		if ( _errorMsgs.length > 0 )
		{
			showErrors();
		}

		// Clean-up
		_errorMsgs.length = 0;
	}

	/**
	 * Add a validation rule. The form Object of the first rule that failed validation will receive
	 * focus when error messages are displayed.
	 *
	 * @param rule (ValidationRule) The validation rule to add.
	 */
	this.addRule = function( rule )
	{
		_rules.push( rule );
	}

	/**
	 * Execute all validation rules.
	 *
	 * @return true if there are no validation errors; false otherwise.
	 */
	this.validateAll = function()
	{
		// Remove and reset error messages.
		removeErrors();
		_errorMsgs.length = 0;
		var focused = false;

		for ( var i = 0; i < _rules.length; i++ )
		{
			var rule = _rules[i];

			if ( !rule.validate() )
			{
				_errorMsgs.push( rule.errorMsg );
				setErrorClass( rule.labelObj, rule.errorClass );

				if (rule.formObjErrorClass != null)
				{
					setFormObjectErrorClass( rule.formObj, rule.formObjErrorClass );
				}
				
				if ( !focused && ( rule.formObj != null ) )
				{
					rule.formObj.focus();
					focused = true;
				}
			}
			else
			{
				unsetErrorClass( rule.labelObj, rule.errorClass );
				if (rule.formObjErrorClass != null)
				{
					unsetFormObjectErrorClass( rule.formObj, rule.formObjErrorClass );
				}
			}
		}

		if ( _errorMsgs.length > 0 )
		{
			showErrors();
			return false;
		}

		return true;
	}

	/**
	 * Set the error class to the form label Object.
	 *
	 * @param formLabelObj (Object) The form label Object.
	 * @param errorClass (String) The error class to set.
	 */
	function setErrorClass( formLabelObj, errorClass )
	{
		var labelClass = formLabelObj.className;

		// If the error class has already been set, then don't bother.
		if ( new RegExp( "^ ?" + errorClass + "$" ).test( labelClass )
			|| ( labelClass.indexOf( " " + errorClass ) != -1 ) )
		{
			return;
		}

		formLabelObj.className += " " + errorClass;
	}
	
	/**
	 * Set the error class to the form Object.
	 *
	 * @param formObj (Object) The form Object.
	 * @param formObjErrorClass (String) The error class to set.
	 */
	function setFormObjectErrorClass( formObj, formObjErrorClass )
	{
		var formObjClass = formObj.className;

		// If the error class has already been set, then don't bother.
		if ( new RegExp( "^ ?" + formObjErrorClass + "$" ).test( formObjClass )
			|| ( formObjClass.indexOf( " " + formObjErrorClass ) != -1 ) )
		{
			return;
		}

		formObj.className += " " + formObjErrorClass;
	}	
	
	/**
	 * Unset the error class from the form label Object.
	 *
	 * @param formLabelObj (Object) The form label Object.
	 * @param errorClass (String) The error class to unset.
	 */
	function unsetErrorClass( formLabelObj, errorClass )
	{
		var labelClass = formLabelObj.className;
		if ( labelClass == "" ) return;

		// For the case where only the error class is set.
		// Need to check for leading space 'cos IE doesn't trim leading space per specification.
		if ( new RegExp( "^ ?" + errorClass + "$" ).test( labelClass ) )
		{
			formLabelObj.className = "";
			return;
		}

		// For the case where more than one class is set.
		formLabelObj.className =
			labelClass.replace( new RegExp( "([\\w ]+) " + errorClass ), "$1" );
	}
	
	/**
	 * Unset the error class from the form Object.
	 *
	 * @param formObj (Object) The form Object.
	 * @param formObjErrorClass (String) The error class to unset.
	 */
	function unsetFormObjectErrorClass( formObj, formObjErrorClass )
	{
		var formObjClass = formObj.className;
		if ( formObjClass == "" ) return;

		// For the case where only the error class is set.
		// Need to check for leading space 'cos IE doesn't trim leading space per specification.
		if ( new RegExp( "^ ?" + formObjErrorClass + "$" ).test( formObjClass ) )
		{
			formObj.className = "";
			return;
		}

		// For the case where more than one class is set.
		formObj.className =
			formObjClass.replace( new RegExp( "([\\w ]+) " + formObjErrorClass), "$1" );
	}
	
	/** Remove all error messages. */
	function removeErrors()
	{
		removeChildren( _errorsSection );
		_errorsSection.className = _hideErrors;
	}

	/**
	 * Remove all children from a node, recursively.
	 *
	 * @param node (Node) The specified node.
	 */
	function removeChildren( node )
	{
		var children = node.childNodes;

		// This works because a child node's index is updated if a lower-indexed sibling is removed.
		while ( children.length > 0 )
		{
			removeChildren( children[0] );
			node.removeChild( children[0] );
		}
	}

	/** Show error messages. */
	function showErrors()
	{
		var tempErrorDiv = document.createElement( "div" );
		var table1 = document.createElement( "table" );				
		table1.setAttribute("width", "100%"); 
		table1.setAttribute("border", "0"); 
		table1.setAttribute("cellspacing", "0"); 
		table1.setAttribute("cellpadding", "0"); 
		
		var tr1 = document.createElement( "tr" );
		table1.appendChild( tr1 );

		var td1 = document.createElement( "td" );
		td1.setAttribute("width", "1%"); 
		td1.setAttribute("valign", "top"); 
		tr1.appendChild( td1 );

		var img = document.createElement( "img" );
		img.setAttribute("src", _messageIconPath); 
		img.setAttribute("alt", getMessage("tooleditor.error.alt.message"));
		img.setAttribute("width", "30");
		img.setAttribute("height", "30");  
		td1.appendChild( img );

		var td2 = document.createElement( "td" );
		tr1.appendChild( td2 );

		var table2 = document.createElement( "table" );
		table2.setAttribute("class", "message-container"); 
		table2.setAttribute("width", "100%"); 
		table2.setAttribute("border", "0"); 
		table2.setAttribute("cellspacing", "0"); 
		table2.setAttribute("cellpadding", "0"); 
		td2.appendChild( table2 );

		var tr21 = document.createElement( "tr" );
		table2.appendChild( tr21 );

		var td3 = document.createElement( "td" );
		td3.setAttribute("class", "message-id"); 

		td3.appendChild( document.createTextNode( _errorInstr ) );
		tr21.appendChild( td3 );
		
		var tr22 = document.createElement( "tr" );
		table2.appendChild( tr22 );
		var td4 = document.createElement( "td" );
		td4.setAttribute("class", "message-description"); 
		tr22.appendChild( td4 );

		var ul = document.createElement( "ul" );
		td4.appendChild( ul );

		for ( var i = 0; i < _errorMsgs.length; i++ )
		{
			var li = document.createElement( "li" );
			ul.appendChild( li );
			li.appendChild( document.createTextNode( _errorMsgs[i] ) );
		}

		tempErrorDiv.appendChild( table1 );
		_errorsSection.innerHTML = tempErrorDiv.innerHTML;
		tempErrorDiv = null;
		_errorsSection.className = _showErrors;

	}
}

/**
 * Template validation rule, to be overridden by specific rules.
 */
function ValidationRule()
{
	/** (Object) Form Object to validate. If not overridden, must be specified in validate(). */
	this.formObj = null;

	/** (Object) Corresponding form label. Must be overridden. */
	this.labelObj = null;

	/** (String) Validation error message. Must be overridden. */
	this.errorMsg = "";

	/** (String) CSS class to append to label on validation failure. May be overridden. */
	this.errorClass = "labelError";

	/** (String) CSS class to append to the input field on validation failure. May be overridden. */
	this.formObjErrorClass = null;

	/**
	 * Validate the form Object. To be overridden by concrete ValidationRule implementation.
	 *
	 * @return true on validation success, false otherwise.
	 */
	this.validate = function()
	{
		return true;
	}
	
	/**
	 * Get the value of the selected option in a Select list.
	 *
	 * @param selectList (Object) The specified Select list.
	 * @return (String) The value of the selected option, or null if none selected.
	 */
	this.getSelectedValue = function( selectList )
	{
		var selectedIndex = selectList.selectedIndex;

		if ( selectedIndex > -1 )
			return selectList.options.item( selectedIndex ).value;

		return null;
	}

	/**
	 * Test if a String is empty, or contains whitespace characters only.
	 *
	 * @param testStr (String) The String to test.
	 * @return true if the String is blank; false otherwise.
	 */
	this.isBlank = function( testStr )
	{
		return /^\s*$/.test( testStr );
	}
}

// -------------------------

/**
 * Controller object to handle the pre-selection of Select lists before submitting a form.
 */
function SubmitListPreselectController()
{
	var _selectLists = new Array();
	
	/**
	 * Add a Select list.
	 *
	 * @param list (String) Select list ID to be added.
	 */
	this.addList = function( listId )
	{
		_selectLists.push( document.getElementById( listId ) );
	}

	/** Select all Options in all added Select lists. */
	this.selectAll = function()
	{
		for ( var i = 0; i < _selectLists.length; i++ )
		{
			var listOpts = _selectLists[i].options;
			for ( var j = 0; j < listOpts.length; j++ )
			{
				listOpts.item( j ).selected = true;
			}
		}
	}
}

// -------------------------

/**
 * Checks if a form is dirty (i.e., modified).
 *
 * @param form (HTMLFormElement) The form to check.
 */
function DirtyFormChecker( form )
{
	var _form = form;
	var _dirty = false;

	/**
	 * Check if the form is dirty.
	 *
	 * @return True if dirty.
	 */
	this.isDirty = function()
	{
		if ( _dirty ) return true;

		for ( var i = 0; i < _form.elements.length; i++ )
		{
			var formElem = _form.elements[i];
			var type = formElem.type;
			if ( isCheckedType( type ) )
			{
				if ( formElem.checked != formElem.defaultChecked )
				{
					return true;
				}
			}
			else if ( isTextType( type ) )
			{
				if ( formElem.value != formElem.defaultValue )
				{
					return true;
				}
			}
			else if ( isSelectType( type ) )
			{
				if (!areMultipleSelected(formElem))
				{
    				var theSelectedIndex = formElem.selectedIndex;
    				if ( theSelectedIndex != undefined && theSelectedIndex >= 0)
    				{
    					var options = formElem.options;
    					if (!options[theSelectedIndex].defaultSelected)
    					{
    						return true;
    					}
    				}
				}
				else
				{
    				var options = formElem.options;
    				for(var i = 0; i < options.length; i++)
    				{
       					if (options[i].selected && !options[i].defaultSelected)
       					{
       						return true;
       					}
    				}
				}
			}
		}

		return false;
	}

	/**
	 * Set the dirty state. This method is for elements whose dirty state cannot be determined by
	 * the isDirty() method.
	 *
	 * @param dirty (boolean) Setting to true affects the entire DirtyFormChecker object, and
	 *        similarly so for false.
	 */
	this.setDirty = function( dirty )
	{
		_dirty = dirty;
	}

	/**
	 * Check if the form element is of checked type (checkbox or radio button).
	 *
	 * @param formType (String) The form type.
	 * @return True if formType is of checked type.
	 */
	function isCheckedType( formType )
	{
		return ( "checkbox" == formType || "radio" == formType );
	}

	/**
	 * Check if the form element is of text type (text, textarea or password).
	 *
	 * @param formType (String) The form type.
	 * @return True if formType is of text type.
	 */
	function isTextType( formType )
	{
		return ( "text" == formType || "textarea" == formType || "password" == formType );
	}
	
	/**
	 * Check if the form element is of select type (select, select-one).
	 *
	 * @param formType (String) The form type.
	 * @return True if formType is of select type.
	 */
	function isSelectType( formType )
	{
		return ( "select-one" == formType || formType.indexOf("select") != -1 );
	}
	
	/**
	 * Checks if there are multiple options selected in the select element.
	 *
	 * @param formElem (Object) The form element.
	 * @return True if there are more than one options selected.
	 */
	function areMultipleSelected( formElem )
	{
		if (!isSelectType( formElem.type))
		{
			return false;
		}
		var options = formElem.options;
		var counter = 0;
		for(var i = 0; i < options.length; i++)
		{
			if (options[i].selected)
			{
				counter++;
			}
		}
		return (counter > 1);
	}
}